import os


class BasicConfig:
    """Configuration for flask instance."""
    SECRET_KEY = os.urandom(24)
    DEBUG = False


class BootstrapConfig:
    """Configuration for Flask-Bootstrap."""
    BOOTSTRAP_SERVE_LOCAL = True

    BOOTSTRAP_ENABLED = True  # used in template


class HttpAuthConfig:
    """Configuration for Flask-BasicAuth."""
    BASIC_AUTH_USERNAME = 'admin'
    BASIC_AUTH_PASSWORD = 'admin'
    BASIC_AUTH_FORCE = True


class IndexBlueprintConfig:
    """Configuration for index blueprint."""
    index_blueprint_prefix = None  # null for non-prefix

    # SCRAPYD_SERVER = 'http://jxltom.me:6800'
    SCRAPYD_SERVER = 'http://127.0.0.1:6800'


class Config:
    """Flask configuration."""

    __slots__ = ('basic', 'bootstrap', 'httpauth', 'index')

    def __init__(self, **kwargs):
        """Initialize configuration.

        :param kwargs: boostrap, httpauth, index.
        """

        # Basic configuration for Flask.
        self.basic = self._dict_from_obj(BasicConfig)

        # Configuration for Flask-Bootstrap.
        if kwargs.get('bootstrap', None):
            self.bootstrap = self._dict_from_obj(BootstrapConfig)

        # Configuration for Flask-BasicAuth.
        if kwargs.get('httpauth', None):
            self.httpauth = self._dict_from_obj(HttpAuthConfig)

        # Configuration for index blueprint.
        if kwargs.get('index', None):
            self.index = self._dict_from_obj(IndexBlueprintConfig)

    @staticmethod
    def _dict_from_obj(obj):
        """Return dictionary from class for configuration."""
        return dict(((k, getattr(obj, k)) for k in dir(obj)))

    def has_attr(self, attr):
        """Check attribute existence."""
        if getattr(self, attr, None) is not None:
            return True
        return False



from flask import Flask
from flask_bootstrap import Bootstrap
from flask_basicauth import BasicAuth

bootstrap = Bootstrap()
httpauth = BasicAuth()


def teardown(func):
    """Teardown works after initialization."""
    _app = None

    def wrapper(*args, **kwargs):
        global _app
        _app = func(*args, **kwargs)
        return _app
    return wrapper


@teardown
def create_app(cfg):
    """Create flask instance from configuration."""

    # Initialize flask instance.
    app = Flask(__name__)
    app.config.update(_upper(cfg.basic))

    # Initialize Flask-Bootstrap.
    _init_bootstrap(app, cfg)

    # Initialize Flask-BasicAuth.
    _init_httpauth(app, cfg)

    # Initialize index blueprint.
    _init_index(app, cfg)

    return app


def _init_bootstrap(app, cfg):
    """Initialize Flask-Bootstrap."""
    if cfg.has_attr('bootstrap'):
        app.config.update(_upper(cfg.bootstrap))
        bootstrap.init_app(app)


def _init_httpauth(app, cfg):
    """Initialize Flask-BasicAuth."""
    if cfg.has_attr('httpauth'):
        app.config.update(_upper(cfg.httpauth))
        httpauth.init_app(app)


def _init_index(app, cfg):
    """Initialize index blueprint."""
    if cfg.has_attr('index'):
        app.config.update(_upper(cfg.index))

        from scrapymon.views.index import index as index_blueprint
        app.register_blueprint(
            index_blueprint, url_prefix=cfg.index['index_blueprint_prefix'])


def _upper(d):
    """Return non-lower dictionary from dictonary."""
    return dict(((k, d[k]) for k in d if k.isupper()))



from collections import OrderedDict
import json

from flask import render_template, flash, current_app, Markup
from werkzeug.local import LocalProxy
import requests

from . import index

# Convenient reference to scrapyd server.
scrapyd_server = LocalProxy(lambda: current_app.config['SCRAPYD_SERVER'])
# Convenient reference to debug setting.
debug = LocalProxy(lambda: current_app.config['DEBUG'])

# Endpoints of scrapyd API.
listprojects_url = '/listprojects.json'
listspiders_url = '/listspiders.json'
listversions_url = '/listversions.json'
listjobs_url = '/listjobs.json'
schedule_url = '/schedule.json'
cancel_url = '/cancel.json'
delproject_url = '/delproject.json'
delversion_url = '/delversion.json'
logs_url = '/logs'


@index.errorhandler(requests.ConnectionError)
def server_connection_error(e):
    """Flash messages if server can not be connected."""
    if debug:
        flash(e.args, 'danger')

    flash(
        'The connection to Scrapyd server can not be established. '
        'Please check status of Scrapyd server in {}.'.format(scrapyd_server),
        'danger'
    )
    return render_template('index/error.html')


@index.route('/')
def projects_dash():
    """Projects view."""
    projects = OrderedDict()

    # Get projects as well as their versions, spiders
    for project in _list_projects():
        spiders = _list_spiders(project)
        versions = _list_versions(project)
        projects[project] = dict(versions=versions, spiders=spiders)

    # Flash hint message when there are no projects.
    if not projects:
        flash(Markup(
            'No available projects currently. Using '
            '<a href="https://github.com/scrapy/scrapyd-client" '
            'target="_blank">scrapyd-client</a> '
            'for uploading projects to Scrapyd server.'),
            'info')

    return render_template('index/projects.html', projects=projects)


@index.route('/jobs')
def jobs_dash():
    """Jobs view."""
    jobs = OrderedDict()
    jobs['pending'], jobs['running'], jobs['finished'] = [], [], []

    # Get jobs and sort them as pending, running and finished.
    for project in _list_projects():
        pending_jobs, running_jobs, finished_jobs = _list_jobs(project)

        for job in pending_jobs:
            job['project'] = project
            jobs['pending'].append(job)

        for job in running_jobs:
            job['project'] = project
            jobs['running'].append(job)

        for job in finished_jobs:
            job['project'] = project
            jobs['finished'].append(job)

    return render_template('index/jobs.html', jobs=jobs)


@index.route('/logs/<project>/<spider>/<job>')
def logs_dash(project, spider, job):
    """Logs view."""
    logs = []

    # Get log from server.
    url = '{}/{}/{}/{}.log'.format(
        scrapyd_server + logs_url, project, spider, job)
    raw = requests.get(url).text

    # Parse response and flash messages.
    if 'file not found' in raw.lower():
        flash(
            'The job {} from spider {} in project {} '
            'has no available logs currently.'.format(job, spider, project),
            'warning'
        )
    elif 'not such resource' in raw.lower():
        flash(
            'Can not get log of job {} from spider {} in project {}.'.format(
                job, spider, project), 'warning'
        )
    else:
        logs = raw.split('\n')

    return render_template('index/logs.html', logs=logs,
                           info=dict(project=project, spider=spider, job=job))


@index.route('/schedule/<project>/<spider>')
def schedule(project, spider):
    """Schedule spider run."""
    # Run spider and get response from server.
    url = scrapyd_server + schedule_url
    raw = requests.post(url, params=dict(project=project, spider=spider)).text
    r = json.loads(raw)

    # Parse response
    status, node_name = r.get('status', ''), r.get('node_name', '')
    jobid = r.get('jobid', '')

    # Flash messages.
    if status != 'ok':
        flash(
            'Can not run spider {} of project {} in node {}. '
            'The raw message returned by Scrapyd server is {}.'.format(
                spider, project, node_name, raw), 'warning'
        )
    else:
        flash(
            'Run spider {} of project {} in node {} successfully. '
            'The job ID is {}.'.format(
                spider, project, node_name, jobid), 'success'
        )

    return 'success'


@index.route('/cancel/<project>/<job>')
def cancel(project, job):
    """Cancel job run."""
    # Cancel job and get response from server.
    url = scrapyd_server + cancel_url
    raw = requests.post(url, params=dict(project=project, job=job)).text
    r = json.loads(raw)

    # Parse response
    status, node_name = r.get('status', ''), r.get('node_name', '')
    prevstate = r.get('prevstate', '')

    # Flash messages.
    if status != 'ok':
        flash(
            'Can not cancle job {} of project {} in node {}. '
            'The raw message returned by Scrapyd server is {}.'.format(
                job, project, node_name, raw), 'warning'
        )
    else:
        flash(
            'Cancel job {} of project {} in node {} successfully. '
            'The previous status is {}.'.format(
                job, project, node_name, prevstate), 'success'
        )

    return 'success'


@index.route('/delete/<project>')
@index.route('/delete/<project>/<version>')
def delproject(project, version=None):
    """Delete project or a version."""
    # Delete project or a specific version and get response from server.
    if version:
        url = scrapyd_server + delversion_url
        raw = requests.post(
            url, params=dict(project=project, version=version)).text
    else:
        url = scrapyd_server + delproject_url
        raw = requests.post(
            url, params=dict(project=project)).text
    r = json.loads(raw)

    # Parse response
    status, node_name = r.get('status', ''), r.get('node_name', '')

    # Flash messages.
    version_ = 'version {}'.format(version) if version else 'all versions'

    if status != 'ok':
        flash(
            'Can not delete project {} with {} in node {}. '
            'The raw message returned by Scrapyd server is {}.'.format(
                project, version_, node_name, raw), 'warning'
        )
    else:
        flash(
            'Delete project {} with {} in node {} successfully.'.format(
                project, version_, node_name), 'success'
        )

    return 'success'


def _list_projects():
    """Get projects list."""
    # Get response from server.
    raw = requests.get(scrapyd_server + listprojects_url).text
    r = json.loads(raw)

    # Parse response.
    status, node_name = r.get('status', ''), r.get('node_name', '')
    projects = r.get('projects', [])

    # Flash error messages.
    if status != 'ok':
        flash(
            'Can not get projects in node {}. '
            'The raw message returned by Scrapyd server is {}.'.format(
                node_name, raw), 'warning'
        )

    # Return projects list.
    return projects


def _list_versions(project):
    """Get versions list of a project"""
    # Get response from server
    raw = requests.get(scrapyd_server + listversions_url,
                       params=dict(project=project)).text
    r = json.loads(raw)

    # Parse response.
    status, node_name = r.get('status', ''), r.get('node_name', '')
    versions = r.get('versions', [])

    # Flash error messages.
    if status != 'ok':
        flash(
            'Can not get versions of project {} in node {}. '
            'The raw message returned by Scrapyd server is {}.'.format(
                project, node_name, raw), 'warning'
        )

    # Return versions list.
    return versions


def _list_spiders(project):
    """Get spiders list of a project"""
    # Get response from server
    raw = requests.get(scrapyd_server + listspiders_url,
                       params=dict(project=project)).text
    r = json.loads(raw)

    # Parse response.
    status, node_name = r.get('status', ''), r.get('node_name', '')
    spiders = r.get('spiders', [])

    # Flash error messages.
    if status != 'ok':
        flash(
            'Can not get spiders of project {} in node {}. '
            'The raw message returned by Scrapyd server is {}.'.format(
                project, node_name, raw), 'warning'
        )

    # Return spiders list.
    return spiders


def _list_jobs(project):
    """Get jobs list of a project."""
    # Get response from server.
    raw = requests.get(scrapyd_server + listjobs_url,
                       params=dict(project=project)).text
    r = json.loads(raw)

    # Parse response.
    status, node_name = r.get('status', ''), r.get('node_name', '')
    pending_jobs = r.get('pending', [])
    running_jobs = r.get('running', [])
    finished_jobs = r.get('finished', [])

    # Flash error messages.
    if status != 'ok':
        flash(
            'Can not get jobs of project {} in node {}. '
            'The raw message returned by Scrapyd server is {}.'.format(
                project, node_name, raw), 'warning'
        )

    return pending_jobs, running_jobs, finished_jobs


@index.route('/_')
def index_test():
    return render_template('base.html')


@index.route('/_auth')
def login_required_test():
    return 'success'



#!/usr/bin/env python

from setuptools import setup, find_packages

from scrapymon import __version__, __author__

setup(
    name='scrapymon',
    version=__version__,
    description='Simple management UI for scrapyd',
    long_description=
    'Go to https://github.com/jxltom/scrapymon/ for more infomation.',
    author=__author__,
    author_email='jxltom@gmail.com',
    url='https://github.com/jxltom/scrapymon/',
    license='MIT',

    include_package_data=True,

    packages=find_packages(),
    install_requires=[
        'flask==0.12',
        'Flask-Bootstrap==3.3.7.0',
        'Flask-BasicAuth==0.2.0',
        'gevent==1.2.1',
        'requests==2.13.0',
    ],

    entry_points={
        'console_scripts': [
            'scrapymon = scrapymon.__main__:main'
        ],
    },

    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
    ],
)



import os
import argparse
import logging

from gevent.pywsgi import WSGIServer

from scrapymon.config import Config
from scrapymon.app import create_app

app = create_app(Config(
    bootstrap=True,
    httpauth=True,
    index=True,
))


def main():
    """Convinent entrypoint for wrapper as script or exe."""
    # Convenient reference
    app_name = 'scrapymon'

    # Customized HelpFomatter without prefix.
    class CustomHelpFormatter(argparse.HelpFormatter):
        def add_usage(self, usage, actions, groups, prefix=''):
            super().add_usage(usage, actions, groups, prefix=prefix)

    # Application usage.
    usage_ = '\n'.join([
        'Usage: {} [--host=<host>] [--port=<port>] '
        '[--server=<address_with_port>] '
        '[--auth=<username:password>'.format(app_name),
        '',
        'Options:',
        '  --host      Default is $HOST or 0.0.0.0 if $HOST is not defined',
        '  --port      Default is $PORT or 5000 if $PORT is not defined',
        '  --server    Scrapyd server address with port. '
        'Default is $SCRAPYD_SERVER  '
        'or http://127.0.0.1:6800 if $SCRAPYD_SERVER is not defined',
        '  --auth      Username and passoword for http basic auth. '
        'Default is $BASIC_AUTH or admin:admin if $BASIC_AUTH is not defined.',
    ])

    # Initialzie argparse.
    argparse_ = argparse.ArgumentParser(
        prog=app_name, usage=usage_, add_help=False,
        formatter_class=CustomHelpFormatter)

    # Add arguments.
    # Default values are relavant environment variables or pre-defined values
    # This may cause problems if one has defined undesired environment variables
    # when running by default.
    argparse_.add_argument('--help', action='store_true', default=False)
    argparse_.add_argument('--host', default=os.environ.get('HOST', '0.0.0.0'))
    # Note that $PORT environment variable has to be used in Heroku.
    # Consider using $DYNO but it is subject to change or removal in Heroku.
    argparse_.add_argument(
        '--port', type=int, default=int(os.environ.get('PORT', 5000)))
    argparse_.add_argument(
        '--server', default=os.environ.get(
            'SCRAPYD_SERVER', 'http://127.0.0.1:6800'))
    argparse_.add_argument('--auth', default=os.environ.get(
            'BASIC_AUTH', 'admin:admin'))

    # Parse options.
    opts = argparse_.parse_args()
    host, port, help_ = opts.host, opts.port, opts.help
    scrapyd_server = opts.server
    basic_auth_username, basic_auth_password = opts.auth.split(':')

    # Print help information
    if help_:
        print(usage_)
        return

    # Update Scrapyd server address.
    app.config.update(SCRAPYD_SERVER=scrapyd_server)

    # Update http basic auth credential.
    app.config.update(BASIC_AUTH_USERNAME=basic_auth_username)
    app.config.update(BASIC_AUTH_PASSWORD=basic_auth_password)

    # Creat server.
    server = WSGIServer((host, port), app)

    # Logging.
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    logging.info('{} is running on http://{}:{}/'.format(
        app_name, server.server_host, server.server_port)
    )

    # Serve forever.
    server.serve_forever()


if __name__ == '__main__':
    # Entrypoint for Heroku/Dokku/Flynn.
    main()
